PROGRAM generate_toeplitz_data

     USE Precision
     USE Timing
     USE Verbose_Module
     USE Construct_Toeplitz
     USE Toeplitz_Module

     IMPLICIT NONE

     INTEGER                       :: n, type, j, capN, N2, nitrefToeplitz
     REAL(KIND=DP)                 :: factoritref, time01, time02
     COMPLEX(KIND=DP), DIMENSION(:), ALLOCATABLE :: a, b, exactx

     time01 = GiveTime()
     time01 = GiveTime()

     OPEN(1,ERR=9910,FILE='toeplitz.input',STATUS='OLD',ACTION='READ')
     OPEN(2,ERR=9920,FILE='toeplitz.dat',STATUS='REPLACE', &
          FORM='UNFORMATTED',ACTION='WRITE')
     OPEN(3,ERR=9930,FILE='exactx.dat',STATUS='REPLACE', &
          FORM='UNFORMATTED',ACTION='WRITE')
!!     OPEN(4,ERR=9940,FILE='generate.time',STATUS='OLD',POSITION='APPEND')
     OPEN(4,ERR=9940,FILE='generate.time',STATUS='REPLACE')

! Process the input.

     READ(1,*) verbose
     READ(1,*) n
     READ(1,*) type
     READ(1,*) nitrefToeplitz
     READ(1,*) factoritref
!!
     READ(1,*) nmatrix

 CLOSE(1)
 OPEN(1,ERR=9910,FILE='toeplitz.input',STATUS='REPLACE',ACTION='WRITE')
     WRITE(1,*) verbose
     WRITE(1,*) n
     WRITE(1,*) type
     WRITE(1,*) nitrefToeplitz
     WRITE(1,*) factoritref
     WRITE(1,*) nmatrix+10

     IF (verbose > 0) THEN
       print*,' '
       print*,'*** BEGIN GENERATE TOEPLITZ DATA ***'
       print*,' '
       print*,'  Input comes from file "toeplitz.input".'
       print*,' '
       print*,'  Toeplitz matrix of type ',type,' of order ',n,&
              ' has to be solved.'
       print*,'  At the end, maximum ', nitrefToeplitz,&
              ' iterative refinement steps will be applied.'
       print*,'  If the relative error is not divided at least by a factor ',&
              factoritref,','
       print*,'  the iterative refinement stops.'
       print*,' '
     END IF

! Construct the Toeplitz matrix.

     ALLOCATE(a(1:2*n-1))

     CALL CToeplitz(n,a,type)

! Construct the exact solution.

     ALLOCATE(exactx(n));

     DO j = 1, n
       exactx(j) = 1.0_DP
     END DO

! Find the smallest power of 2 that is larger than or equal to n.
!! can be programmed in another way using log2(n)
     capN = 1
     DO WHILE (capN < n)
       capN = capN * 2
     END DO

     N2 = capN*2

! Calculate the right-hand side.

     ALLOCATE(b(n))

     b = Tx(a,exactx,N2)

     WRITE(3) exactx

     DEALLOCATE(exactx)

     WRITE(2) verbose
     WRITE(2) n
     WRITE(2) N2
     WRITE(2) a
     WRITE(2) b
     WRITE(2) nitrefToeplitz
     WRITE(2) factoritref

     DEALLOCATE(a)
     DEALLOCATE(b)

     IF (verbose > 0) THEN
       print*,'  Toeplitz-data written on file "toeplitz.dat"'
       print*,'  exact x       written on file "exactx.dat"'
       print*,' '
     END IF

     time02 = GiveTime()

     IF (verbose > 0) THEN
       print*,'  Execution time = ',time02-time01
       print*,' '
       print*,'*** END GENERATE TOEPLITZ DATA ***'
       print*,' '
     END IF

     WRITE(4,8010) time02-time01
8010 FORMAT(ES9.2)

     CLOSE(4)
     CLOSE(3)
     CLOSE(2)
     CLOSE(1)

     GOTO 1000

9910 STOP 'File "toeplitz.input" does not exist.'
9920 STOP
9930 STOP
9940 STOP

1000 END PROGRAM generate_toeplitz_data
