     PROGRAM Transform_RIP_T_Main
!-----------------------------------------------------------------------
!
! The solution of the interpolation problem is transformed into
! a solution for the Toeplitz-problem.
! The file "outratint.dat" is needed as input.
!
!-----------------------------------------------------------------------
     USE Precision
     USE Timing
     USE Verbose_Module
     USE Toeplitz_Module

     IMPLICIT NONE

     INTEGER       :: n, nitrefToeplitz, N2
     REAL(KIND=DP) :: factoritref
     REAL(KIND=DP), DIMENSION(:), ALLOCATABLE    :: errorv
     REAL(KIND=DP) :: time01, time02, time03, time04
     COMPLEX(KIND=DP), DIMENSION(:), ALLOCATABLE :: a, b, x, exactx

!-----------------------------------------------------------------------
     time01 = GiveTime()
     time01 = GiveTime()
     OPEN(1,FILE='error.txt',STATUS='REPLACE')
!!     OPEN(1,FILE='error.txt',STATUS='OLD',POSITION='APPEND')
     OPEN(2,FILE='residualT.txt',STATUS='REPLACE')
!!     OPEN(2,FILE='residualT.txt',STATUS='OLD',POSITION='APPEND')
     OPEN(4,FILE='toeplitz.dat',STATUS='OLD',FORM='UNFORMATTED',ACTION='READ')
     OPEN(90,FILE='relresidual.txt',STATUS='REPLACE')
!!     OPEN(90,FILE='relresidual.txt',STATUS='OLD',POSITION='APPEND')
     OPEN(92,FILE='exactx.dat',STATUS='OLD',FORM='UNFORMATTED',ACTION='READ')
     OPEN(93,FILE='x.dat',STATUS='REPLACE',FORM='UNFORMATTED',ACTION='WRITE')
!!     OPEN(94,FILE='transformRIPT.time',STATUS='OLD',POSITION='APPEND')
     OPEN(94,FILE='transformRIPT.time',STATUS='REPLACE')

! Read the input.

     READ(4) verbose
     READ(4) n
     READ(4) N2
     ALLOCATE(a(1:2*n-1)); ALLOCATE(b(n))
     READ(4) a
     READ(4) b
     READ(4) nitrefToeplitz
     READ(4) factoritref


     IF (verbose > 0) THEN
       print*,' '
       print*,'*** BEGIN TRANSFORM RIP-SOLUTION TO TOEPLITZ-SOLUTION ***'
       print*,' '
       print*,'  The file "outratint.dat" is needed as input.'
       print*,' '
       print*,'  The solution of the interpolation problem is transformed'
       print*,'  into the solution for the Toeplitz-problem.'
       print*,' '
     END IF

! Compute the solution "x".

     ALLOCATE(x(n))
     OPEN(12,FILE='outratint.dat',STATUS='OLD',ACTION='READ',FORM='UNFORMATTED')

     time02 = GiveTime()

         CALL Transform_RIP_T(n,a,b,x,nref=nitrefToeplitz,&
                              factor=factoritref)
     time03 = GiveTime()

     CLOSE(12)

     WRITE(93) x
  
     ALLOCATE(exactx(n))

     READ(92) exactx

! Error statistics

     ALLOCATE(errorv(n))
     errorv=abs(x-exactx) ! Component-wise absolute error
     WRITE(1,9001) sum(errorv)/n, minval(errorv), maxval(errorv) 
!                  mean           minimum         maximum
     errorv = abs(b-Tx(a,x,2*n)) ! Component-wise absolute residual error
     WRITE(2,9001) sum(errorv)/n, minval(errorv), maxval(errorv) 
!                  mean           minimum         maximum
     WRITE(90,9001) sum(errorv)/sum(abs(b)), minval(errorv)/sum(abs(b))*n, &
                    maxval(errorv)/sum(abs(b))*n
!               Relative to the right-hand side
     DEALLOCATE(errorv)
     DEALLOCATE(exactx)


    DEALLOCATE(a); DEALLOCATE(b); DEALLOCATE(x)

    time04 = GiveTime()

     IF (verbose > 0) THEN
       print*,'  Execution time of subroutine Transform_RIP_T = ',time03-time02
       print*,'  Execution time of Main Program = ',time04-time01
       print*,' '
       print*,'*** END TRANSFORM RIP-SOLUTION TO TOEPLITZ-SOLUTION ***'
       print*,' '
     END IF

    WRITE(94,8010) time03-time02, time04-time01

    CLOSE(94)
    CLOSE(93)
    CLOSE(92)
    CLOSE(90)
    CLOSE(4)
    CLOSE(2)
    CLOSE(1)

4000 FORMAT (I10)
8010 FORMAT (2ES9.2)
9000 FORMAT (ES24.16,ES24.16)
9001 FORMAT (ES24.16,ES24.16,ES24.16)

     END PROGRAM Transform_RIP_T_Main
!
!-----------------------------------------------------------------------
!
